library(shiny)

black <- paste("<font color=\"black\">")
red <- paste("<font color=\"red\">")            
blue <- paste("<font color=\"blue\">")
nruns <- 10^5

shinyServer(function(input, output,session) {

  
   data1 <- reactive({
      if(input$p=="Fair coin p=0.5") p<-0.5
      if(input$p=="Slightly unfair coin p=0.6") p<-0.6
      if(input$p=="Quite unfair coin p=0.7") p<-0.7
      for(i in 1:input$step) 
        return(sample(c("H","T"),size=100,replace=T,prob=c(p,1-p)))
    })

   data2 <- reactive({
   if(input$p=="Fair coin p=0.5") p<-0.5
      if(input$p=="Slightly unfair coin p=0.6") p<-0.6
      if(input$p=="Quite unfair coin p=0.7") p<-0.7
      for(i in 1:input$step) 
        return(rbinom(nruns,100,p))
    })


   output$table1 <- renderText({
       for(i in 1:input$step) {
           x <- data1()
           xt <- table(x)           
           line <- "<h4>Sequence of Flips</h4>"
           line[2] <- paste(x,collapse=" ")
           line[3] <- "<hr>"
           line[4] <- "<h4>Table of Counts</h4>"
           line[5] <- "<h4>heads &nbsp;&nbsp;&nbsp; tails</h4>"
           line[6] <- paste("<h4>",xt[1],"&nbsp;&nbsp;",xt[2],"</h4>")
           if(diff(input$lim)>2) {
                line[7] <- paste("<h5>Chosen Rejection Limits: Low=",input$lim[1]," High=",input$lim[2],"</h5>")                 
                line[8] <- paste("<h5>",blue,input$lim[1]," &le; ",xt[1]," &le; ",input$lim[2]," so coin looks fair</h5>")
                if(xt[1]<=input$lim[1])
                    line[8] <- paste("<h5>",red,xt[1],"&lt;",input$lim[1]," so coin not fair</h5>")
                if(xt[1]>=input$lim[2])
                    line[8] <- paste("<h5>",red,xt[1],"&gt;",input$lim[2]," so coin not fair</h5>")
           }         
           line
           
       }    
       line   
   })    
   
   table2 <- reactive({
       x <- table(data2())
       n <- length(x)
       cols <- rep(black,n)
       if(diff(input$lim)>2) {
            cols <- rep(blue,n)
            k <- c(1:n)[as.numeric(names(x))<input$lim[1]]
            cols[k] <- red
            k <- c(1:n)[as.numeric(names(x))>input$lim[2]]
            cols[k] <- red            
       }     
       ncols <- 10
       k <- floor(n/ncols)
       line <- "<table>"
       for(i in 1:k) {
            txt <- c("<tr><th>Number of Heads</th>","<tr><th>Counts</th>")
            for(l in 1:ncols) {
                txt[1] <- paste(txt[1],"<td>",names(x)[(i-1)*ncols+l],"</td><td>&nbsp;&nbsp;</td>")
                txt[2] <- paste(txt[2],"<td>",cols[(i-1)*ncols+l],x[(i-1)*ncols+l],"</td><td>&nbsp;&nbsp;</td>")
            }
            txt[1] <- paste(txt[1],"</tr>")
            txt[2] <- paste(txt[2],"</tr>")
            line <- c(line,txt)  
            if(ncols*i<n) line <- c(line,"<tr><td colspan = 2*ncols+1>&nbsp;</td></tr>")
       } 
       if(ncols*k<n) {
            txt <- c("<tr><th>Number of Heads</th>","<tr><th>Counts</th>")
            for(l in (ncols*k+1):n) {
                txt[1] <- paste(txt[1],"<td>",names(x)[l],"</td><td>&nbsp;&nbsp;</td>")
                txt[2] <- paste(txt[2],cols[l],"<td>",x[l],"</td><td>&nbsp;&nbsp;</td>")
            }
            txt[1] <- paste(txt[1],"</tr>")
            txt[2] <- paste(txt[2],"</tr>")
            line <- c(line,txt)  
       }       
       line <- c(line,"</table>") 
       if(diff(input$lim)>2) {
            z <- as.numeric(names(x))
            a <- sum(x[z<input$lim[1]])+sum(x[z>input$lim[2]])
            a <- round(100*a/nruns,1)
            line <- c(line,paste("<h4>Percentage of runs that lead to conclusion that coin is not fair: ",a,"%</h4>",sep=""))
       }     
       line    
           
   })
   
   output$text2 <- renderText({
        txt <- paste("<h4>Number of heads of 100000 runs</h4>")
        c(txt,table2())
   
   })     
   output$plot1 <- renderPlot({
        x<-data2()
        z <- range(as.numeric(names(table(x))))
        hist(x,breaks=seq(z[1]-0.5,z[2]+0.5,1), main="Number of Heads",)
        if(diff(input$lim)>2) abline(v=input$lim,lwd=2,col="red")
   })

   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#coin\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 5/15/2015</h6>")
      
  })       
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates some concepts of hypothesis testing using the example of coin tossing</h5>"
        
  })

})

